// Load shared default patterns first
const loadPatterns = () => {
  chrome.storage.local.get('patterns', (data) => {
    let patternStrings = data.patterns;

    if (!patternStrings || patternStrings.length === 0) {
      // Save defaults on first run
      chrome.storage.local.set({ patterns: DEFAULT_PATTERNS }, () => {
        runScanner(DEFAULT_PATTERNS);
      });
    } else {
      runScanner(patternStrings);
    }
  });
};

const runScanner = (patternStrings) => {
  const patterns = patternStrings.map(p => {
    try {
      return new RegExp(p, 'i');
    } catch (e) {
      console.warn("Invalid regex skipped:", p);
      return null;
    }
  }).filter(Boolean);

  const text = document.body.innerText;
  const doc = nlp(text);
  const flagged = [];
  const matchCounts = new Map();

  doc.sentences().forEach(s => {
    const sentenceText = s.text();
    for (let pattern of patterns) {
      if (pattern.test(sentenceText)) {
        flagged.push(sentenceText);

        const key = pattern.toString();
        matchCounts.set(key, (matchCounts.get(key) || 0) + 1);
        const count = matchCounts.get(key);

        highlightText(sentenceText, pattern, count);
        break;
      }
    }
  });
  // After scanning is complete:
  const totalMatches = flagged.length;

  if (totalMatches > 0) {
    showMatchCount(totalMatches);
  }
};

loadPatterns();

// highlightText and <style> injection code.
function highlightText(sentence, pattern, count) {
  const words = sentence.trim().split(/\s+/);
  const chunks = [];

  for (let i = 0; i < words.length - 3; i++) {
    const chunk = words.slice(i, i + 4).join(" ");
    chunks.push(chunk);
  }

  const treeWalker = document.createTreeWalker(
    document.body,
    NodeFilter.SHOW_TEXT,
    {
      acceptNode: (node) => {
        const parent = node.parentNode;
        if (
          parent &&
          parent.nodeName === "MARK"
        ) {
          return NodeFilter.FILTER_REJECT;
        }
        if (
          parent &&
          getComputedStyle(parent).display !== 'none' &&
          getComputedStyle(parent).visibility !== 'hidden' &&
          !['SCRIPT', 'STYLE', 'NOSCRIPT'].includes(parent.nodeName)
        ) {
          return NodeFilter.FILTER_ACCEPT;
        }
        return NodeFilter.FILTER_REJECT;
      }
    },
    false
  );
  

  while (treeWalker.nextNode()) {
    const node = treeWalker.currentNode;
    const text = node.nodeValue;

    for (let chunk of chunks) {
      const regex = new RegExp(chunk.replace(/[.*+?^${}()|[\]\\]/g, '\\$&'), "i");
      const match = text.match(regex);
      if (match) {
        const span = document.createElement("mark");
        span.textContent = match[0];

        span.addEventListener("click", (e) => {
          e.stopPropagation();
          showTooltip(e.target, pattern, count);
        });

        const parts = text.split(regex);
        const parent = node.parentNode;
        if (!parent) return; // skip if parent is null (safety check)

        const fragment = document.createDocumentFragment();

        parts.forEach((part, index) => {
          fragment.appendChild(document.createTextNode(part));
          if (index < parts.length - 1) {
            const copy = span.cloneNode(true);
            copy.addEventListener("click", (e) => {
              e.stopPropagation();
              showTooltip(e.target, pattern, count);
            });
            fragment.appendChild(copy);
          }
        });

        parent.replaceChild(fragment, node);
        // Do NOT return — allow continued scanning.
      }
    }
  }
}
// Tooltip style.
// Tooltip creation
const tooltip = document.createElement("div");
tooltip.style.position = "absolute";
tooltip.style.background = "#fff";
tooltip.style.border = "1px solid #ccc";
tooltip.style.padding = "8px 12px";
tooltip.style.borderRadius = "6px";
tooltip.style.boxShadow = "0 2px 6px rgba(0,0,0,0.2)";
tooltip.style.fontSize = "0.85rem";
tooltip.style.zIndex = "9999";
tooltip.style.maxWidth = "300px";
tooltip.style.display = "none";
document.body.appendChild(tooltip);

// Hide tooltip on outside click
document.addEventListener("click", () => {
  tooltip.style.display = "none";
});

function showTooltip(target, pattern, count) {
  tooltip.innerHTML = `
    <strong>Matched by pattern:</strong><br/>
    <code>${pattern}</code><br/>
    <span style="font-size: 0.85em; color: #555;">
      This pattern matched <strong>${count}</strong> time${count === 1 ? "" : "s"} on this page.
    </span>
  `;

  const rect = target.getBoundingClientRect();
  tooltip.style.left = `${window.scrollX + rect.left}px`;
  tooltip.style.top = `${window.scrollY + rect.bottom + 6}px`;
  tooltip.style.display = "block";
}
function showMatchCount(count) {
  const badge = document.createElement("div");
  badge.textContent = `🟡 ${count} potential match${count === 1 ? "" : "es"}`;
  badge.style.position = "fixed";
  badge.style.bottom = "16px";
  badge.style.right = "16px";
  badge.style.background = "#fff8dc";
  badge.style.border = "1px solid #e0c97f";
  badge.style.color = "#333";
  badge.style.padding = "8px 12px";
  badge.style.borderRadius = "8px";
  badge.style.boxShadow = "0 2px 6px rgba(0,0,0,0.15)";
  badge.style.fontSize = "0.9rem";
  badge.style.zIndex = "10000";

  document.body.appendChild(badge);
}



// Style <mark> tags
const style = document.createElement("style");
style.textContent = `
  mark {
    background: #ffef96;
    color: black;
    padding: 0 2px;
    border-radius: 2px;
  }
`;
document.head.appendChild(style);
